package com.yihu.ehr.resource.service;

import com.yihu.ehr.query.BaseJpaService;
import com.yihu.ehr.redis.client.RedisClient;
import com.yihu.ehr.resource.dao.RsDictionaryEntryDao;
import com.yihu.ehr.resource.model.RsDictionary;
import com.yihu.ehr.resource.model.RsDictionaryEntry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

/**
 * @author linaz
 * @created 2016.05.17 16:33
 */
@Service
@Transactional
public class RsDictionaryEntryService extends BaseJpaService<RsDictionaryEntry, RsDictionaryEntryDao> {

    @Autowired
    protected JdbcTemplate jdbcTemplate;
    @Autowired
    protected RedisClient redisClient;
    @Autowired
    private RsDictionaryEntryDao dictionaryEntryDao;

    @Autowired
    private RsDictionaryService rsDictionaryService;

    /**
     * 简化资源字典缓存，默认缓存3600秒
     *
     * @param dict
     * @param code
     * @return
     */
    public String getCacheValue(String dict, String code) {
        String key = "rs:dict-entry:" + dict + ":" + code;
        String value = redisClient.get(key);
        if (value != null) {
            return value;
        }

        RsDictionary rsDictionary = rsDictionaryService.findByCode(dict);
        if (rsDictionary == null) {
            return "";
        }

        RsDictionaryEntry rsDictionaryEntry;
        if (rsDictionary.getDynamic() == 0) {
            rsDictionaryEntry = dictionaryEntryDao.findByDictCodeAndCode(dict, code);
        } else {
            String sql = "select code, name from (" + rsDictionary.getSql() + ") a where a.code=?";
            rsDictionaryEntry = jdbcTemplate.queryForObject(
                    sql, new Object[]{code},
                    new BeanPropertyRowMapper<>(RsDictionaryEntry.class));
        }

        if (rsDictionaryEntry == null) {
            return "";
        }

        redisClient.set(key, rsDictionaryEntry.getName(), 3600);


        return rsDictionaryEntry.getName();
    }

    /**
     * 获取所有字典项
     *
     * @param dict
     * @return
     */
    public List<String> getCacheEntries(String dict) {
        String key = "rs:dict-entry:" + dict + ":*";
        Set<String> keys = redisClient.keys(key);
        RsDictionary rsDictionary = rsDictionaryService.findByCode(dict);
        if (rsDictionary == null) {
            return new ArrayList<>();
        }

        List<RsDictionaryEntry> entries = new ArrayList<>();
        if (rsDictionary.getDynamic() == 0) {
            entries = dictionaryEntryDao.findByDictCode(dict);
        } else {
            String sql = "select count(1) from (" + rsDictionary.getSql() + ") a";
            Integer size = jdbcTemplate.queryForObject(sql, (resultSet, i) -> resultSet.getInt(1));

            if (keys.size() != size) {
                sql = "select code, name from (" + rsDictionary.getSql() + ") a";
                entries = jdbcTemplate.query(sql, new BeanPropertyRowMapper(RsDictionaryEntry.class));
            }
        }

        HashMap<String, Serializable> map = new HashMap<>(entries.size());
        ArrayList<String> list = new ArrayList<>();
        if (keys.size() != entries.size() && entries.size() != 0) {
            entries.forEach(rsDictionaryEntry -> {
                String codeKey = "rs:dict-entry:" + dict + ":" + rsDictionaryEntry.getCode();
                list.add(rsDictionaryEntry.getCode());
                keys.add(codeKey);
                map.put(codeKey, rsDictionary.getName());
            });

            redisClient.multiSetData(map);
        } else {
            keys.forEach(value -> {
                list.add(value.replace("rs:dict-entry:" + dict + ":", ""));
            });
        }

        return list;
    }

    /**
     * 清空资源字典缓存，资源字典更新时需要调用
     *
     * @param dict
     * @return
     */
    public void deleteCacheValue(String dict) {
        String key = "rs:dict-entry:" + dict + ":*";
        Set<String> keys = redisClient.keys(key);
        redisClient.delete(keys);
    }


    public RsDictionaryEntry findById(int id) {
        return dictionaryEntryDao.findOne(id);
    }

    public List<RsDictionaryEntry> findByDictCode(String code) {
        return dictionaryEntryDao.findByDictCode(code);
    }

    public int countByDictId(int dictId) {

        return dictionaryEntryDao.countByDictId(dictId);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public RsDictionaryEntry insert(RsDictionaryEntry model) {
        KeyHolder keyHolder = new GeneratedKeyHolder();
        jdbcTemplate.update(new PreparedStatementCreator() {
            @Override
            public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
                PreparedStatement pst = connection.prepareStatement(
                        "INSERT INTO rs_dictionary_entry(dict_code, code, name, description, dict_id) VALUES(?,?,?,?,?)",
                        Statement.RETURN_GENERATED_KEYS);
                pst.setString(1, model.getDictCode());
                pst.setString(2, model.getCode());
                pst.setString(3, model.getName());
                pst.setString(4, model.getDescription());
                pst.setInt(5, model.getDictId());
                return pst;
            }
        }, keyHolder);
        model.setId(keyHolder.getKey().intValue());
        return model;
    }
}
